var Base64 = window.Base64;
var _ = window._;

var provide = function (apiName, timeout) {
    return function () {
        var args = _.toArray(arguments);
        var callback = args.length > 0 ? args.pop() : function () {};
        if (typeof callback !== 'function') {
            args.push(callback);
            callback = function () {};
        }

        var asyncParams = {
            arguments: args
        };

        var callbackName = 'asyncCallback' + _.uniqueId();

        var timer = null;
        if (timeout !== -1) {
            timer = setTimeout(function () {
                window[callbackName] = null;

                callback('TIMEOUT');
            }, timeout);
        }
        window[callbackName] = function (b64Result) {
            if (timer) {
                clearTimeout(timer);
            }

            window[callbackName] = null;

            var ret = JSON.parse(Base64.decode(b64Result));
            callback.apply(null, ret);
        };

        asyncParams.callback = callbackName;

        _.defer(function () {
            var encodedArgs = Base64.encode(JSON.stringify(asyncParams));
            if (window.JSInterface) {
                window.JSInterface[apiName](encodedArgs);
            } else {
                var iframe = document.createElement('iframe');
                iframe.style.display = 'none';
                iframe.src = 'async:' + apiName + ':' + encodedArgs;
                document.body.appendChild(iframe);
                document.body.removeChild(iframe);
            }
        });
    };
};

var images;
var imgSrcMap = {};
var formulaSrcMap = {};

function getImageSrc() {
  images = $('img.yst-image-loading');
  images.on('load', function () {
    // remove loading class
    $(this).removeClass('yst-image-loading');
  });

  svgDetect();
  images.each(function (index, img) {
    img = $(img);
    var attr, srcMap;

    if (img.attr('flag') === 'tex') {
      // is formula
      attr = img.attr('data-src');
      if (attr && attr.lastIndexOf) {
        attr = attr.lastIndexOf('?') > 0 ? attr.concat('&fontSize=34') : attr.concat('?fontSize=34');
        img.addClass('yst-latex');
        srcMap = formulaSrcMap;
      } else {
        return;
      }
    } else if (img.attr('data-formula')) {
      // generate image src for formula
      attr = 'http';
      if (window.qstSafeProtocol) {
        attr = 'https';
      }

      attr += '://gallery.fbcontent.cn/latex?decode=false&fontSize=34&latex=' +
        encodeURIComponent(img.attr('data-formula'));
      img.attr('data-src', attr).addClass('yst-latex');
      srcMap = formulaSrcMap;
    } else if (img.attr('data-src')) {
      attr = img.attr('data-src');
      srcMap = imgSrcMap;
    } else {
      return;
    }

    if (srcMap[attr]) {
      srcMap[attr].push(index);
    } else {
      srcMap[attr] = [index];
    }
  });
  setImages(Object.keys(formulaSrcMap))
  setImages(Object.keys(imgSrcMap))
  var mulitImg = document.querySelectorAll('img')
      var imgTotal = mulitImg.length
      var flag = 0
       for(var i = 0 ; i < imgTotal ; i++){
          var img = new Image()
          img.src = mulitImg[i].src
          img.onload = function(){
             //第i张图片加载完成
             flag++
             if( flag == imgTotal ){
                 setTimeout(function() {
                  provide('updateHeight', -1)();
                 })

             }
          }
          img.onerror = function() {
            flag++
            if( flag == imgTotal ){
                setTimeout(function() {
                    provide('updateHeight', -1)();
                })

            }
          }
       }
}

  // pass src to client
function svgDetect() {
  // if broswer supports svg
  var svgSupport = document.implementation.hasFeature('http://www.w3.org/TR/SVG11/feature#BasicStructure', '1.1');
  //if not supports, convert svg to jpeg
  $('img[data-src$=".svg"]').each(function () {
    var svg = $(this);
    if (!svgSupport) {
      var dimension = this.getBoundingClientRect();
      svg.attr('src',
        svg.attr('data-src') + '?convertFormat=jpg&width=' + dimension.width + '&height=' + dimension.height);
    } else {
      svg.attr('src', svg.attr('data-src'));
    }
    svg.removeAttr('data-src');
  });
}

window.setImages = function (sources) {
  $('table').removeAttr("width")
  var list = document.getElementsByTagName('table')
  for (var i = 0; i < list.length; i++) {
    list[i].style.maxWidth = '100%'
  }
  sources.forEach(function (src) {
    var mark = src.match(/http|https/);
    (imgSrcMap[src] || formulaSrcMap[src]).forEach(function (index) {
      var image = $(images[index]);
      // if formula, set width and height
      if (src.match(/https?:\/\/(gallery\.fbcontent\.cn|ytkgallery\.yuantiku\.ws)\/latex/) || image.attr('class').match(/half-image/)) {
        var tmpImg = $('<img>');
        tmpImg.on('load', function () {
          image.css({
            width: (this.naturalWidth / 200) + 'rem'
          });
          $(this).remove();

          image.attr('src', src);
        });

        tmpImg.attr('src', src);
      } else {
        // set images src
        image.attr('src',  src);
        // use rem
        image.on('load', function() {
          image.css({
            width: (this.width / 100) + 'rem'
          })
        })
      }
    });
  });
  dragElements();
};

window.setBody = function (b64Str) {
  images = null;
  imgSrcMap = {};
  formulaSrcMap = {};
  var content = Base64.decode(b64Str)
  
  if (navigator.userAgent.match(/iPhone/) || navigator.userAgent.match(/iPad/)) {
    content = content.replace(/http:/g, 'https:');
    window.qstSafeProtocol = true;
  }
  $('body').html(content
    .replace(/(src[\s]?=[^<>\/]+?|src="https?:\/\/(gallery\.fbcontent\.cn|ytkgallery\.yuantiku\.ws)\/latex\?[\s\S]*?)(\/?>)/g, 'data-$1$3')
    .replace(/(data-src|data-formula)/g, 'class="yst-image-loading" $1')
    .replace(/(img[^>]*?class="[^>"]*?)("[^>]*?)(class="yst-image-loading")/g, '$1 yst-image-loading$2')
    .replace(/(img[^>]*?)(\sclass="yst-image-loading")([^>]*?)(class="[^>"]*?)/g, '$1$3$4yst-image-loading ')
    .replace(/(font-size:\s*)([^0-9.])([0-9]{2})(px)/g, '$1$20.$3rem') // use rem
  ).promise().done(getImageSrc);
}

window.setFontSize = function (b64Str) {
  var fontSize = Base64.decode(b64Str)
  var html = document.querySelectorAll('html')[0]
  html.style.fontSize = fontSize + 'px'
}
// img.width > window.width  drag scroll
function dragElements() {
  // make large images and tables that cause overflow-x draggable
  var winWidth = window.screen.width,
    winPadding = parseInt($('.section-content, .ios-section-content').css('paddingLeft')) || 10;
  if (navigator.userAgent.indexOf('Android') >= 0 && window.devicePixelRatio) {
    winWidth /= window.devicePixelRatio;
  }

  function addDragContainer() {
    var elem = $(this);
    var minX = winWidth - elem.width() - winPadding; //some tables may resize after loading
    if (minX < 0) {
      if (this.parentNode && this.parentNode.classList && this.parentNode.classList.contains('drag-container')) {
        return
      }
      var p = document.createElement('p');
      p.style.overflowX = 'scroll';
      p.style.overflowY = 'hidden';
      p.classList.add('drag-container');
      this.parentNode.insertBefore(p, this);
      p.appendChild(this);
    }
    return false
  }

  function draggable() {
    var elem = this
    if (elem.width) {
      addDragContainer.call(elem)
    } else {
      this.addEventListener('touchstart', addDragContainer)
    }
  }

  $('span[style]').filter(function () { // img in koba
    if (this.style.width) {
      var width = parseInt(this.style.width);
      return width && (winWidth - winPadding < width);
    }
    return false;
  }).each(draggable);
}

$(function () {
  provide('documentReady', -1)();
});